/*
 * Copyright (C) 2014 Canonical, Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: James Henstridge <james.henstridge@canonical.com>
 */

#ifndef SCOPES_SCOPE_H
#define SCOPES_SCOPE_H

#include <unity/scopes/SearchReply.h>
#include <unity/scopes/ScopeBase.h>
#include <unity/scopes/Variant.h>

class ScopesScope : public unity::scopes::ScopeBase
{
    friend class ScopesQuery;
public:
    virtual int start(std::string const&, unity::scopes::RegistryProxy const&) override;
    virtual void stop() override;

    virtual unity::scopes::QueryBase::UPtr create_query(unity::scopes::Query const &q, unity::scopes::SearchMetadata const& hints) override;
    virtual unity::scopes::QueryBase::UPtr preview(unity::scopes::Result const& result, unity::scopes::ActionMetadata const& hints) override;

    virtual unity::scopes::ActivationBase::UPtr activate(unity::scopes::Result const &result, unity::scopes::ActionMetadata const& hints) override;
    virtual unity::scopes::ActivationBase::UPtr perform_action(unity::scopes::Result const &result, unity::scopes::ActionMetadata const& hints, std::string const &widget_id, std::string const &action_id) override;

private:
    unity::scopes::RegistryProxy registry;

    unity::scopes::ScopeProxy online_scope;
};

class ScopesQuery : public unity::scopes::SearchQuery
{
public:
    ScopesQuery(ScopesScope &scope, unity::scopes::Query const& query);
    virtual void cancelled() override;
    virtual void run(unity::scopes::SearchReplyProxy const&reply) override;

private:
    const ScopesScope &scope;
    const unity::scopes::Query query;
};

class ScopesPreview : public unity::scopes::PreviewQuery
{
public:
    ScopesPreview(ScopesScope &scope, unity::scopes::Result const& result);
    virtual void cancelled() override;
    virtual void run(unity::scopes::PreviewReplyProxy const& reply) override;

private:
    const ScopesScope &scope;
    const unity::scopes::Result result;
};

class ScopesActivation : public unity::scopes::ActivationBase
{
public:
    ScopesActivation(unity::scopes::Result const &result);
    virtual unity::scopes::ActivationResponse activate() override;
private:
    const unity::scopes::Result result;
};


#endif
